import { NavigationOption, ImageSource, Color, BarStyle, TitleAlignment, BarButtonItem, TitleItem } from './typing';
export interface ShadowImage {
    image?: ImageSource;
    color?: Color;
}
export interface Style {
    screenBackgroundColor?: Color;
    topBarStyle?: BarStyle;
    topBarColor?: Color;
    topBarColorDarkContent?: Color;
    topBarColorLightContent?: Color;
    statusBarColorAndroid?: Color;
    navigationBarColorAndroid?: Color;
    hideBackTitleIOS?: boolean;
    elevationAndroid?: number;
    shadowImage?: ShadowImage;
    backIcon?: ImageSource;
    topBarTintColor?: Color;
    topBarTintColorDarkContent?: Color;
    topBarTintColorLightContent?: Color;
    titleTextColor?: Color;
    titleTextColorDarkContent?: Color;
    titleTextColorLightContent?: Color;
    titleTextSize?: number;
    titleAlignmentAndroid?: TitleAlignment;
    barButtonItemTextSize?: number;
    swipeBackEnabledAndroid?: boolean;
    tabBarColor?: Color;
    tabBarShadowImage?: ShadowImage;
    tabBarItemColor?: Color;
    tabBarUnselectedItemColor?: Color;
    tabBarBadgeColor?: Color;
}
export declare type TabBarStyle = Pick<Style, 'tabBarColor' | 'tabBarShadowImage' | 'tabBarItemColor' | 'tabBarUnselectedItemColor'>;
export interface TabBadge {
    index: number;
    text?: string;
    hidden: boolean;
    dot?: boolean;
}
export interface TabIcon {
    index: number;
    icon: ImageSource;
    unselectedIcon?: ImageSource;
}
declare type Nullable<T> = {
    [P in keyof T]: T[P] extends T[P] | undefined ? T[P] | null : T[P];
};
export declare class Garden {
    sceneId: string;
    static setStyle(style?: Style): void;
    constructor(sceneId: string);
    setLeftBarButtonItem(buttonItem: Nullable<BarButtonItem> | null): void;
    setRightBarButtonItem(buttonItem: Nullable<BarButtonItem> | null): void;
    setTitleItem(titleItem: TitleItem): void;
    updateOptions(options: NavigationOption): void;
    updateTabBar(options: TabBarStyle): void;
    setTabIcon(icon: TabIcon | TabIcon[]): void;
    setTabBadge(badge: TabBadge | TabBadge[]): void;
    setMenuInteractive(enabled: boolean): void;
}
export {};
