import { RouteGraph, Route } from './router';
import { Visibility } from './hooks';
import { IndexType, ResultType, NavigationInterceptor, Layout, NavigationItem, PropsType } from './typing';
interface Params {
    animated?: boolean;
    moduleName?: string;
    layout?: Layout;
    popToRoot?: boolean;
    requestCode?: number;
    props?: IndexType;
    options?: NavigationItem;
    [index: string]: any;
}
interface NavigationState {
    params: {
        readonly [index: string]: any;
    };
    resultListeners: ResultListener<any>[];
}
interface ResultListener<T extends ResultType> {
    (requestCode: number, resultCode: number, data: T): void;
    cancel: () => void;
}
export declare class Navigator {
    sceneId: string;
    moduleName?: string | undefined;
    static of(sceneId: string): Navigator;
    static find(moduleName: string): Promise<Navigator | undefined>;
    static current(): Promise<Navigator>;
    static currentRoute(): Promise<Route>;
    static routeGraph(): Promise<RouteGraph[]>;
    static setRoot(layout: Layout, sticky?: boolean): Promise<void>;
    static setRootLayoutUpdateListener(willSetRoot?: () => void, didSetRoot?: () => void): void;
    static dispatch(sceneId: string, action: string, params?: Params): Promise<boolean>;
    static setInterceptor(interceptFn: NavigationInterceptor): void;
    visibility: Visibility;
    constructor(sceneId: string, moduleName?: string | undefined);
    readonly state: NavigationState;
    setParams(params: {
        [index: string]: any;
    }): void;
    dispatch(action: string, params?: Params): Promise<boolean>;
    result(requestCode: number, resultCode: number, data: ResultType): void;
    unmount(): void;
    private waitResult;
    push<T extends ResultType, P extends PropsType = PropsType>(moduleName: string, props?: P, options?: NavigationItem): Promise<[number, T]>;
    pushLayout<T extends ResultType>(layout: Layout): Promise<[number, T]>;
    pop(): Promise<boolean>;
    popTo(moduleName: string): Promise<boolean>;
    popToRoot(): Promise<boolean>;
    redirectTo<P extends PropsType = PropsType>(moduleName: string, props?: P, options?: NavigationItem): Promise<boolean>;
    isStackRoot(): Promise<boolean>;
    present<T extends ResultType, P extends PropsType = PropsType>(moduleName: string, props?: P, options?: NavigationItem, requestCode?: number): Promise<[number, T]>;
    presentLayout<T extends ResultType>(layout: Layout, requestCode?: number): Promise<[number, T]>;
    dismiss(): Promise<boolean>;
    showModal<T extends ResultType, P extends PropsType = PropsType>(moduleName: string, props?: P, options?: NavigationItem, requestCode?: number): Promise<[number, T]>;
    showModalLayout<T extends ResultType>(layout: Layout, requestCode?: number): Promise<[number, T]>;
    hideModal(): Promise<boolean>;
    setResult<T extends ResultType>(resultCode: number, data?: T): void;
    switchTab(index: number, popToRoot?: boolean): Promise<boolean>;
    toggleMenu(): Promise<boolean>;
    openMenu(): Promise<boolean>;
    closeMenu(): Promise<boolean>;
    signalFirstRenderComplete(): void;
}
export {};
