import { AppRegistry } from 'react-native';
import React, { useEffect, useCallback } from 'react';
import { Navigator } from './Navigator';
import { NavigationModule, EventEmitter, EVENT_NAVIGATION, KEY_SCENE_ID, KEY_ON, ON_COMPONENT_APPEAR, ON_COMPONENT_DISAPPEAR, } from './NavigationModule';
import { Garden } from './Garden';
import { router } from './router';
import store from './store';
import { bindBarButtonItemClickEvent, removeBarButtonItemClickEvent } from './utils';
import { useResult } from './hooks';
function getDisplayName(WrappedComponent) {
    return WrappedComponent.displayName || WrappedComponent.name || 'Component';
}
function withNavigator(moduleName) {
    return function (WrappedComponent) {
        function FC(props, ref) {
            const { sceneId } = props;
            const navigator = Navigator.of(sceneId);
            if (navigator.moduleName === undefined) {
                navigator.moduleName = moduleName;
            }
            const garden = new Garden(sceneId);
            useEffect(() => {
                navigator.signalFirstRenderComplete();
                return () => {
                    removeBarButtonItemClickEvent(navigator.sceneId);
                    store.removeNavigator(navigator.sceneId);
                    navigator.unmount();
                };
            }, [navigator]);
            useEffect(() => {
                const subscription = EventEmitter.addListener(EVENT_NAVIGATION, data => {
                    if (navigator.sceneId === data[KEY_SCENE_ID]) {
                        if (data[KEY_ON] === ON_COMPONENT_APPEAR) {
                            navigator.visibility = 'visible';
                        }
                        else if (data[KEY_ON] === ON_COMPONENT_DISAPPEAR) {
                            navigator.visibility = 'gone';
                        }
                    }
                });
                return () => {
                    subscription.remove();
                };
            }, [navigator]);
            const resultCallback = useCallback((requestcode, resultCode, data) => {
                navigator.result(requestcode, resultCode, data);
            }, [navigator]);
            useResult(sceneId, resultCallback);
            const injected = {
                garden,
                navigator,
            };
            return <WrappedComponent ref={ref} {...props} {...injected}/>;
        }
        const FREC = React.forwardRef(FC);
        FREC.displayName = `withNavigator(${getDisplayName(WrappedComponent)})`;
        return FREC;
    };
}
let wrap;
export class ReactRegistry {
    static startRegisterComponent(hoc) {
        store.clear();
        wrap = hoc;
        ReactRegistry.registerEnded = false;
        NavigationModule.startRegisterReactComponent();
    }
    static endRegisterComponent() {
        if (ReactRegistry.registerEnded) {
            console.warn(`Please don't call ReactRegistry#endRegisterComponent multiple times.`);
            return;
        }
        ReactRegistry.registerEnded = true;
        NavigationModule.endRegisterReactComponent();
    }
    static registerComponent(appKey, getComponentFunc, routeConfig) {
        if (routeConfig) {
            router.registerRoute(appKey, routeConfig);
        }
        const WrappedComponent = getComponentFunc();
        // build static options
        let options = bindBarButtonItemClickEvent(WrappedComponent.navigationItem) || {};
        NavigationModule.registerReactComponent(appKey, options);
        let RootComponent;
        if (wrap) {
            RootComponent = wrap(withNavigator(appKey)(WrappedComponent));
        }
        else {
            RootComponent = withNavigator(appKey)(WrappedComponent);
        }
        AppRegistry.registerComponent(appKey, () => RootComponent);
    }
}
export function withNavigationItem(item) {
    return function (Func) {
        ;
        Func.navigationItem = item;
        return Func;
    };
}
