import { Insets } from 'react-native';
import { Navigator } from './Navigator';
export interface IndexType {
    [index: string]: any;
}
export interface PropsType {
    [index: string]: any;
}
export declare type ResultType = IndexType | null;
interface Extras {
    sceneId: string;
    index?: number;
}
export interface NavigationInterceptor {
    (action: string, extras: Extras): boolean | Promise<boolean>;
}
export declare type Color = string;
export declare type ImageSource = {
    uri: string;
    scale?: number;
    height?: number;
    width?: number;
};
export declare const BarStyleLightContent = "light-content";
export declare type BarStyleLightContent = typeof BarStyleLightContent;
export declare const BarStyleDarkContent = "dark-content";
export declare type BarStyleDarkContent = typeof BarStyleDarkContent;
export declare type BarStyle = BarStyleLightContent | BarStyleDarkContent;
export declare const TitleAlignmentLeft = "left";
export declare type TitleAlignmentLeft = typeof TitleAlignmentLeft;
export declare const TitleAlignmentCenter = "center";
export declare type TitleAlignmentCenter = typeof TitleAlignmentCenter;
export declare type TitleAlignment = TitleAlignmentCenter | TitleAlignmentLeft;
export declare const LayoutFittingExpanded = "expanded";
export declare type LayoutFittingExpanded = typeof LayoutFittingExpanded;
export declare const LayoutFittingCompressed = "compressed";
export declare type LayoutFittingCompressed = typeof LayoutFittingCompressed;
export declare type LayoutFitting = LayoutFittingExpanded | LayoutFittingCompressed;
export interface NavigationOption {
    passThroughTouches?: boolean;
    statusBarHidden?: boolean;
    statusBarColorAndroid?: Color;
    topBarStyle?: BarStyle;
    topBarColor?: Color;
    topBarShadowHidden?: boolean;
    topBarAlpha?: number;
    topBarTintColor?: Color;
    titleTextColor?: Color;
    titleTextSize?: number;
    navigationBarColorAndroid?: Color;
    backButtonHidden?: boolean;
    backInteractive?: boolean;
}
export interface NavigationItem extends NavigationOption {
    screenBackgroundColor?: Color;
    topBarHidden?: boolean;
    extendedLayoutIncludesTopBar?: boolean;
    swipeBackEnabled?: boolean;
    titleItem?: TitleItem;
    leftBarButtonItem?: BarButtonItem;
    rightBarButtonItem?: BarButtonItem;
    leftBarButtonItems?: BarButtonItem[];
    rightBarButtonItems?: BarButtonItem[];
    backItemIOS?: BackItem;
    tabItem?: TabItem;
}
export interface BarButtonItem {
    title?: string;
    icon?: ImageSource;
    insetsIOS?: Insets;
    action?: (navigator: Navigator) => void;
    enabled?: boolean;
    tintColor?: Color;
    renderOriginal?: boolean;
}
export interface TitleItem {
    title?: string;
    moduleName?: string;
    layoutFitting?: LayoutFitting;
}
export interface BackItem {
    title: string;
    tintColor?: Color;
}
export interface TabItem {
    title: string;
    icon?: ImageSource;
    unselectedIcon?: ImageSource;
    hideTabBarWhenPush?: boolean;
}
export interface Layout {
    [index: string]: {};
}
export interface Screen extends Layout {
    screen: {
        moduleName: string;
        props?: IndexType;
        options?: NavigationItem;
    };
}
export interface Stack extends Layout {
    stack: {
        children: Layout[];
        options?: {};
    };
}
export interface Tabs extends Layout {
    tabs: {
        children: Layout[];
        options?: {
            selectedIndex?: number;
            tabBarModuleName?: string;
            sizeIndeterminate?: boolean;
        };
    };
}
export interface Drawer extends Layout {
    drawer: {
        children: [Layout, Layout];
        options?: {
            maxDrawerWidth?: number;
            minDrawerMargin?: number;
            menuInteractive?: boolean;
        };
    };
}
export {};
